<?php

namespace Drupal\terms_export_import\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\terms_export_import\Service\TermsExportImportService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TermsImportForm. Provides the import form for terms.
 */
class TermsImportForm extends FormBase {

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\terms_export_import\Service\TermsExportImportService
   */
  protected TermsExportImportService $exportImportService;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): TermsImportForm {
    $instance = parent::create($container);
    $instance->exportImportService = $container->get('terms_export_import.service');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'terms_import_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['vocabulary'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Vocabulary'),
      '#target_type' => 'taxonomy_vocabulary',
      '#required' => TRUE,
      '#description' => $this->t('Select the vocabulary to import into.'),
    ];

    $form['import_file'] = [
      '#type' => 'file',
      '#title' => $this->t('CSV file'),
      '#description' => $this->t('Upload a CSV file with columns: name,description,parent,weight'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import CSV'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $file = file_save_upload('import_file', [
      'file_validate_extensions' => ['csv'],
    ]);
    if (!$file) {
      $form_state->setErrorByName('import_file', $this->t('Please upload a valid CSV file.'));
    }
    else {
      // Move file to temporary location so can process it.
      $destination = 'public://taxonomy_import_export/' . $file->getFilename();
      $file->setPermanent();
      $file->save();
      $file->move('public://taxonomy_import_export', $file->getFilename());
      $form_state->setValue('uploaded_file', $file);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\file\FileInterface $file */
    $file = $form_state->getValue('uploaded_file');
    if (!$file) {
      $this->messenger()->addError($this->t('File upload error.'));
      return;
    }
    $vocabulary_id = $form_state->getValue('vocabulary');
    $vocab = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->load($vocabulary_id);
    if (!$vocab) {
      $this->messenger()->addError($this->t('Invalid vocabulary selected.'));
      return;
    }

    // Read CSV file content.
    $file_uri = $file->getFileUri();
    $csv_string = file_get_contents($file_uri);

    // Parse CSV into array.
    $terms_data = $this->exportImportService->parseCsv($csv_string);

    // Import into taxonomy.
    $this->exportImportService->importTerms($vocab->id(), $terms_data);

    $this->messenger()->addStatus($this->t('Taxonomy terms have been successfully imported.'));
  }

}
